import Foundation

public class EventoService {
    private let BASE_API = "http://localhost:8000/api/v1"
    
    public static let shared = EventoService()
    
    public enum APIErro: Error {
        case error(_ errorString: String)
    }
    
    public func getParticipacoes<T: Decodable>(jwt:String,opcao: String, completion: @escaping (Result<T,APIErro>) -> Void) {
        let url = URL(string: "\(BASE_API)/evento/\(opcao)")!
        var request = URLRequest(url: url)
        request.setValue("application/json", forHTTPHeaderField: "Content-Type")
        request.setValue(jwt, forHTTPHeaderField: "jwt")
        request.httpMethod = "GET"
        
        URLSession.shared.dataTask(with: request) { (data,response,erro) in
            let httpResponse = response as? HTTPURLResponse
            let statusCode = httpResponse?.statusCode
            
            
            if statusCode! >= 500 {
                completion(.failure(.error("Erro no servidor, tente novamente")))
                return
            }
            
            else if statusCode! >= 400 {
                completion(.failure(.error("Não autenticado")))
                return
            }
            
            else if statusCode! >= 201 {
                completion(.failure(.error("Vazio")))
                return
            }
            
            
            do {
                
                guard let data = data else {
                    completion(.failure(.error("Dados corrompidos. Tente novamente")))
                    return
                }
                
                let dados = try JSONDecoder().decode(T.self, from: data)
                completion(.success(dados))
                return
            }
            catch  {
                print(error)
                completion(.failure(.error("Dados corrompidos. Tente novamente")))
                return
            }
            
        }.resume()
        
    }
    
    public func adicionarEvento<T: Decodable>(jwt: String, titulo: String , descricao: String,link: String, data: Date, total_assentos_linha: Int , total_assentos_coluna: Int , imagem: Data,assentos_bloqueados: [Int],  completion: @escaping (Result<T,APIErro>) -> Void ) {
        
        let formatter = DateFormatter()
        //formatter.dateFormat = "yyyy-MM-dd'T'HH:mm:ss"
        formatter.timeZone = TimeZone(abbreviation: "BRT")
        formatter.dateFormat = "yyyy-MM-dd'T'HH:mm:ssZ"
        let longDateVersion = formatter.string(from:  data)
        
        
        let request = FormDataRequest(url: "\(BASE_API)/evento",jwt: jwt)
        
        request.addDataField(name: "imagem", fileName: "imagem.png", data: imagem, mimeType: "img/png")
        request.addTextField(name: "titulo", value: titulo)
        request.addTextField(name: "descricao", value: descricao)
        request.addTextField(name: "data", value: longDateVersion)
        request.addTextField(name: "link", value: link)
        request.addTextField(name: "total_assentos_linha", value: String(total_assentos_linha))
        request.addTextField(name: "total_assentos_coluna", value: String(total_assentos_coluna))
        request.addArrayField(name: "assentos_bloqueados", value: assentos_bloqueados)
        
        URLSession.shared.dataTask(with: request) { (data,response,error) in
            let httpResponse = response as? HTTPURLResponse
            let statusCode = httpResponse?.statusCode
            
            if statusCode! >= 500 {
                completion(.failure(.error("Erro no servidor, tente novamente")))
                return
            }
            
            else if statusCode! >= 400 {
                completion(.failure(.error("Não autorizado")))
                return
            }
            
            
            
            do {
                guard let data = data else {
                    completion(.failure(.error("Dados corrompidos. Tente novamente")))
                    return
                }
                
                let dados = try JSONDecoder().decode(T.self, from: data)
                completion(.success(dados))
                return
            }
            catch  {
                completion(.failure(.error("Dados corrompidos. Tente novamente")))
                return
            }
            
        }.resume()
    }
    
    public func getEvento<T: Decodable>(jwt:String, codigo: String ,completion: @escaping (Result<T,APIErro>) -> Void) {
        let url = URL(string: "\(BASE_API)/evento/\(codigo)")!
        var request = URLRequest(url: url)
        request.setValue("application/json", forHTTPHeaderField: "Content-Type")
        request.setValue(jwt, forHTTPHeaderField: "jwt")
        request.httpMethod = "GET"
        
        URLSession.shared.dataTask(with: request) { (data,response,erro) in
            let httpResponse = response as? HTTPURLResponse
            let statusCode = httpResponse?.statusCode
            
            
            if statusCode! >= 500 {
                completion(.failure(.error("Erro no servidor, tente novamente")))
                return
            }
            
            else if statusCode! == 404 {
                completion(.failure(.error("Nenhum evento encontrado")))
                return
            }
            
            else if statusCode! >= 400 {
                completion(.failure(.error("Não autenticado")))
                return
            }
            
            else if statusCode! >= 201 {
                completion(.failure(.error("Nenhum evento encontrado")))
                return
            }
            
            
            do {
                
                guard let data = data else {
                    completion(.failure(.error("Dados corrompidos. Tente novamente")))
                    return
                }
                
                let dados = try JSONDecoder().decode(T.self, from: data)
                completion(.success(dados))
                return
            }
            catch  {
                print(error)
                completion(.failure(.error("Dados corrompidos. Tente novamente")))
                return
            }
            
        }.resume()
        
    }
    
    
    public func apagarEvento<T: Decodable>(jwt:String, id: Int ,completion: @escaping (Result<T,APIErro>) -> Void) {
        let url = URL(string: "\(BASE_API)/evento/\(id)")!
        var request = URLRequest(url: url)
        request.setValue("application/json", forHTTPHeaderField: "Content-Type")
        request.setValue(jwt, forHTTPHeaderField: "jwt")
        request.httpMethod = "DELETE"
        
        URLSession.shared.dataTask(with: request) { (data,response,erro) in
            let httpResponse = response as? HTTPURLResponse
            let statusCode = httpResponse?.statusCode
            
            
            if statusCode! >= 500 {
                completion(.failure(.error("Erro no servidor, tente novamente")))
                return
            }
            
            else if statusCode! == 404 {
                completion(.failure(.error("Nenhum evento encontrado")))
                return
            }
            
            else if statusCode! >= 400 {
                completion(.failure(.error("Não autenticado")))
                return
            }
            
            else if statusCode! >= 201 {
                completion(.failure(.error("Nenhum evento encontrado")))
                return
            }
            
            
            do {
                
                guard let data = data else {
                    completion(.failure(.error("Dados corrompidos. Tente novamente")))
                    return
                }
                
                let dados = try JSONDecoder().decode(T.self, from: data)
                completion(.success(dados))
                return
            }
            catch  {
                print(error)
                completion(.failure(.error("Dados corrompidos. Tente novamente")))
                return
            }
            
        }.resume()
        
    }
    
    public func consultarParticipacoes<T: Decodable>(jwt:String, id: Int ,completion: @escaping (Result<T,APIErro>) -> Void) {
        let url = URL(string: "\(BASE_API)/evento/participacoes/\(id)")!
        var request = URLRequest(url: url)
        request.setValue("application/json", forHTTPHeaderField: "Content-Type")
        request.setValue(jwt, forHTTPHeaderField: "jwt")
        request.httpMethod = "GET"
        
        URLSession.shared.dataTask(with: request) { (data,response,erro) in
            let httpResponse = response as? HTTPURLResponse
            let statusCode = httpResponse?.statusCode
            
            
            if statusCode! >= 500 {
                completion(.failure(.error("Erro no servidor, tente novamente")))
                return
            }
            
            else if statusCode! == 404 {
                completion(.failure(.error("Nenhum evento encontrado")))
                return
            }
            
            else if statusCode! >= 400 {
                completion(.failure(.error("Não autenticado")))
                return
            }
            
            else if statusCode! >= 201 {
                completion(.failure(.error("Vazio")))
                return
            }
            
            
            do {
                
                guard let data = data else {
                    completion(.failure(.error("Dados corrompidos. Tente novamente")))
                    return
                }
                
                let dados = try JSONDecoder().decode(T.self, from: data)
                completion(.success(dados))
                return
            }
            catch  {
                print(error)
                completion(.failure(.error("Dados corrompidos. Tente novamente")))
                return
            }
            
        }.resume()
        
    }
    
}
