import Foundation

public class UsuarioService {
    private let BASE_API = "http://localhost:8000/api/v1"
    
    public static let shared = UsuarioService()
    
    public enum APIErro: Error {
        case error(_ errorString: String)
    }
    
    public func getPerfil<T: Decodable>(_ jwt: String , completion: @escaping (Result<T,APIErro>) -> Void ) {
        let url = URL(string: "\(BASE_API)/usuario")!
        var request = URLRequest(url: url)
        request.setValue("application/json", forHTTPHeaderField: "Content-Type")
        request.setValue(jwt, forHTTPHeaderField: "jwt")
        request.httpMethod = "GET"
        
        URLSession.shared.dataTask(with: request) { (data,response,error) in
            let httpResponse = response as? HTTPURLResponse
            let statusCode = httpResponse?.statusCode
            
            
            if statusCode! >= 500 {
                completion(.failure(.error("Erro no servidor, tente novamente")))
                return
            }
            
            else if statusCode! >= 400 {
                completion(.failure(.error("Não autorizado")))
                return
            }
            
            
            do {
                
                guard let data = data else {
                    completion(.failure(.error("Dados corrompidos. Tente novamente")))
                    return
                }
                
                let dados = try JSONDecoder().decode(T.self, from: data)
                completion(.success(dados))
                return
            }
            catch _ {
                completion(.failure(.error("Dados corrompidos. Tente novamente")))
                return
            }
            
        }.resume()
    }
    
    public func atualizarDados<T: Decodable>(_ jwt: String, _ nome: String, _ email: String , completion: @escaping (Result<T,APIErro>) -> Void ) {
        let url = URL(string: "\(BASE_API)/usuario")!
        var request = URLRequest(url: url)
        let parametros = [ "nome": nome, "email" : email ]
        let body = try! JSONSerialization.data(withJSONObject: parametros)
        request.setValue("application/json", forHTTPHeaderField: "Content-Type")
        request.setValue(jwt, forHTTPHeaderField: "jwt")
        request.httpMethod = "PUT"
        request.httpBody = body
        
        URLSession.shared.dataTask(with: request) { (data,response,error) in
            let httpResponse = response as? HTTPURLResponse
            let statusCode = httpResponse?.statusCode
            
            
            if statusCode! >= 500 {
                completion(.failure(.error("Erro no servidor, tente novamente")))
                return
            }
            
            else if statusCode! == 401 {
                completion(.failure(.error("Email já cadastrado")))
                return
            }
            
            
            do {
                
                guard let data = data else {
                    completion(.failure(.error("Dados corrompidos. Tente novamente")))
                    return
                }
                
                let dados = try JSONDecoder().decode(T.self, from: data)
                completion(.success(dados))
                return
            }
            catch _ {
                completion(.failure(.error("Dados corrompidos. Tente novamente")))
                return
            }
            
        }.resume()
    }
    
    public func atualizarSenha<T: Decodable>(_ jwt: String, _ senhaAtual: String, _ novaSenha: String , completion: @escaping (Result<T,APIErro>) -> Void ) {
        let url = URL(string: "\(BASE_API)/usuario/redefinir")!
        var request = URLRequest(url: url)
        let parametros = [ "senhaAtual": senhaAtual, "novaSenha" : novaSenha ]
        let body = try! JSONSerialization.data(withJSONObject: parametros)
        request.setValue("application/json", forHTTPHeaderField: "Content-Type")
        request.setValue(jwt, forHTTPHeaderField: "jwt")
        request.httpMethod = "POST"
        request.httpBody = body
        
        URLSession.shared.dataTask(with: request) { (data,response,error) in
            let httpResponse = response as? HTTPURLResponse
            let statusCode = httpResponse?.statusCode
            
            
            if statusCode! >= 500 {
                completion(.failure(.error("Erro no servidor, tente novamente")))
                return
            }
            
            else if statusCode! == 401 {
                completion(.failure(.error("Email já cadastrado")))
                return
            }
            
            
            do {
                
                guard let data = data else {
                    completion(.failure(.error("Dados corrompidos. Tente novamente")))
                    return
                }
                
                let dados = try JSONDecoder().decode(T.self, from: data)
                completion(.success(dados))
                return
            }
            catch _ {
                completion(.failure(.error("Dados corrompidos. Tente novamente")))
                return
            }
            
        }.resume()
    }
    
    public func alterarFoto<T: Decodable>(jwt: String,imagem: Data,completion: @escaping (Result<T,APIErro>) -> Void ) {
        
        let request = FormDataRequest(url: "\(BASE_API)/usuario/foto",jwt: jwt)
        
        request.addDataField(name: "imagem", fileName: "imagem.png", data: imagem, mimeType: "img/png")
        
        URLSession.shared.dataTask(with: request) { (data,response,error) in
            let httpResponse = response as? HTTPURLResponse
            let statusCode = httpResponse?.statusCode
            
            if statusCode! >= 500 {
                completion(.failure(.error("Erro no servidor, tente novamente")))
                return
            }
            
            else if statusCode! >= 400 {
                completion(.failure(.error("Não autorizado")))
                return
            }
            
            
            
            do {
                guard let data = data else {
                    completion(.failure(.error("Dados corrompidos. Tente novamente")))
                    return
                }
                
                let dados = try JSONDecoder().decode(T.self, from: data)
                completion(.success(dados))
                return
            }
            catch  {
                completion(.failure(.error("Dados corrompidos. Tente novamente")))
                return
            }
            
        }.resume()
    }
    
    
}
